@@ -53,7 +53,9 @@ callOptim <- function(func, method, nD, control, lowbnd, uppbnd){
   } else if(method == "Nelder-Mead"){
     res <- optim(getStart(nD), fn=func, control = control)
   } else if(method == "solnp"){ # no need for transformed values for solnp
-    require(Rsolnp, quietly = TRUE)
+    avail <- requireNamespace("Rsolnp", quietly = TRUE)
+    if(!avail)
+      stop("Need suggested package Rsolnp for this calculation to use solnp optimizer")
     ## get starting value (need feasible starting value for solnp)
     ## try whether equal allocation is feasible
     eq <- rep(1/nD, nD)
@@ -70,8 +72,8 @@ callOptim <- function(func, method, nD, control, lowbnd, uppbnd){
     }
     con <- list(trace = 0)
     con[(namc <- names(control))] <- control
-    res <- solnp(strt, fun=func, eqfun=eqfun, eqB=1,
-                 control = con, LB = lowbnd, UB = uppbnd)
+    res <- Rsolnp::solnp(strt, fun=func, eqfun=eqfun, eqB=1,
+                         control = con, LB = lowbnd, UB = uppbnd)
   } 
   res
 }
@@ -155,7 +157,7 @@ optDesign <- function(models, probs, doses,
       stop("n needs to be of length 1")
   }
   if(missing(Delta)){
-    if(substr(designCrit, 1, 3) == "TD")
+    if(designCrit %in% c("TD", "Dopt&TD"))
       stop("need to specify target difference \"Delta\"")
   } else {
     if(Delta <= 0)
@@ -183,14 +185,17 @@ optDesign <- function(models, probs, doses,
     stop("standDopt needs to contain a logical value")
   standInt <- as.integer(standDopt) # use standardized or non-stand. D-optimality
   nD <- length(doses)
-  if(designCrit == "TD" | designCrit == "TD&Dopt"){ # check whether TD exists in (0,max(dose))
+  if(designCrit == "TD" | designCrit == "Dopt&TD"){ # check whether TD exists in (0,max(dose))
+    if(length(unique(direction)) > 1)
+      stop("need to provide either \"increasing\" or \"decreasing\" as direction to optDesign, when TD optimal designs should be calculated")
+    direction <- unique(direction)
     tdMods <- TD(models, Delta, "continuous", direction)
     tdMods[tdMods > max(doses)] <- NA
     if(any(is.na(tdMods)))
       stop("TD does not exist for ",
            paste(names(tdMods)[is.na(tdMods)], collapse=", " ), " model(s)")
   }
-  if(designCrit == "Dopt" | designCrit == "TD&Dopt"){ # check whether Fisher matrix can be singular
+  if(designCrit == "Dopt" | designCrit == "Dopt&TD"){ # check whether Fisher matrix can be singular
     np <- nPars(names(models))
     if(max(np) > length(doses))
       stop("need at least as many dose levels as there are parameters to calculate Dopt design.")
@@ -330,14 +335,17 @@ calcCrit <- function(design, models, probs, doses,
     stop("need to specify clinical relevance parameter")
   direction <- attr(models, "direction")
 
-  if(designCrit == "TD" | designCrit == "TD&Dopt"){ # check whether TD exists in (0,max(dose))
+  if(designCrit == "TD" | designCrit == "Dopt&TD"){ # check whether TD exists in (0,max(dose))
+    if(length(unique(direction)) > 1)
+      stop("need to provide either \"increasing\" or \"decreasing\" as direction to optDesign, when TD optimal designs should be calculated")
+    direction <- unique(direction)
     tdMods <- TD(models, Delta, "continuous", direction)
     tdMods[tdMods > max(doses)] <- NA
     if(any(is.na(tdMods)))
       stop("TD does not exist for ",
            paste(names(tdMods)[is.na(tdMods)], collapse=", " ), " model(s)")
   }
-  if(designCrit == "Dopt" | designCrit == "TD&Dopt"){ # check whether Fisher matrix can be singular
+  if(designCrit == "Dopt" | designCrit == "Dopt&TD"){ # check whether Fisher matrix can be singular
     np <- nPars(names(models))
     if(max(np) > length(doses))
       stop("need more dose levels to calculate Dopt design.")
@@ -369,13 +377,14 @@ calcCrit <- function(design, models, probs, doses,
   res <- numeric(nrow(design))
   ## check for sufficient number of design points
   iter <- 1:nrow(design)
-  count <- apply(design, 1, function(x) sum(x > 0.0001))
+  design0 <- sweep(design, 2, nold, "+")
+  count <- apply(design0, 1, function(x) sum(x > 0.0001))
   ind <- count < max(p[probs > 0])
   if(any(ind)){
     iter <- iter[!ind]
     res[ind] <- NA
     if(all(is.na(res)))
-      warning("need more at least as many dose levels in the design as parameters in the model")
+      warning("need at least as many dose levels in the design as parameters in the model")
   }
   for(i in iter){
     res[i] <- optFunc(design[i,], xvec=as.double(lst$modgrads),
